@@ -27,7 +27,8 @@
 
 namespace hip {
 
-static amd::Monitor eventSetLock{"Guards global event set"};
+// Guards global event set
+static amd::Monitor eventSetLock{};
 static std::unordered_set<hipEvent_t> eventSet;
 
 bool Event::ready() {
@@ -193,7 +194,9 @@ hipError_t Event::enqueueStreamWaitCommand(hipStream_t stream, amd::Command* com
 }
 
 hipError_t Event::streamWait(hipStream_t stream, uint flags) {
-  hip::Stream* hip_stream = hip::getStream(stream);
+  // Get the stream without any resolution
+  constexpr bool kWait = false;
+  hip::Stream* hip_stream = hip::getStream(stream, kWait);
   // Access to event_ object must be lock protected
   amd::ScopedLock lock(lock_);
   if ((event_ == nullptr) || (event_->command().queue() == hip_stream) || ready()) {
@@ -236,7 +239,9 @@ hipError_t Event::recordCommand(amd::Command*& command, amd::HostQueue* stream,
 // ================================================================================================
 hipError_t Event::enqueueRecordCommand(hipStream_t stream, amd::Command* command, bool record) {
   command->enqueue();
-  if (event_ == &command->event()) return hipSuccess;
+  if (event_ == &command->event()) {
+    return hipSuccess;
+  }
   if (event_ != nullptr) {
     event_->release();
   }
@@ -249,7 +254,9 @@ hipError_t Event::enqueueRecordCommand(hipStream_t stream, amd::Command* command
 // ================================================================================================
 hipError_t Event::addMarker(hipStream_t stream, amd::Command* command,
                             bool record, bool batch_flush) {
-  hip::Stream* hip_stream = hip::getStream(stream);
+  // Skip wait as we should not be resolving stream in this sub
+  constexpr bool kWait = false;
+  hip::Stream* hip_stream = hip::getStream(stream, kWait);
   // Keep the lock always at the beginning of this to avoid a race. SWDEV-277847
   amd::ScopedLock lock(lock_);
   hipError_t status = recordCommand(command, hip_stream, 0, batch_flush);
@@ -406,9 +413,7 @@ hipError_t hipEventRecord_common(hipEvent_t event, hipStream_t stream) {
         "[hipGraph] Current capture node EventRecord on stream : %p, Event %p", stream, event);
     s->SetCaptureEvent(event);
     std::vector<hip::GraphNode*> lastCapturedNodes = s->GetLastCapturedNodes();
-    if (!lastCapturedNodes.empty()) {
-      e->SetNodesPrevToRecorded(lastCapturedNodes);
-    }
+    e->SetNodesPrevToRecorded(lastCapturedNodes);
   } else {
     if (g_devices[e->deviceId()]->devices()[0] != &hip_stream->device()) {
       return hipErrorInvalidHandle;
@@ -464,7 +469,7 @@ hipError_t ihipEventQuery(hipEvent_t event) {
   if ((hip_stream != nullptr && hip_stream != hipStreamLegacy) &&
       (s->GetCaptureStatus() == hipStreamCaptureStatusActive)) {
     s->SetCaptureStatus(hipStreamCaptureStatusInvalidated);
-    return HIP_RETURN(hipErrorCapturedEvent);
+    HIP_RETURN(hipErrorCapturedEvent);
   }
   return e->query();
 }
