@@ -55,13 +55,20 @@ namespace hip {
 int GraphNode::nextID = 0;
 int Graph::nextID = 0;
 std::unordered_set<GraphNode*> GraphNode::nodeSet_;
-amd::Monitor GraphNode::nodeSetLock_{"Guards global node set"};
+// Guards global node set
+amd::Monitor GraphNode::nodeSetLock_{};
 std::unordered_set<Graph*> Graph::graphSet_;
-amd::Monitor Graph::graphSetLock_{"Guards global graph set"};
+// Guards global graph set
+amd::Monitor Graph::graphSetLock_{};
 std::unordered_set<GraphExec*> GraphExec::graphExecSet_;
-amd::Monitor GraphExec::graphExecSetLock_{"Guards global exec graph set"};
+// Guards global exec graph set
+// we have graphExec object as part of child graph and we need recursive lock
+amd::Monitor GraphExec::graphExecSetLock_(true);
 std::unordered_set<UserObject*> UserObject::ObjectSet_;
-amd::Monitor UserObject::UserObjectLock_{"Guards global user object"};
+// Guards global user object
+amd::Monitor UserObject::UserObjectLock_{};
+// Guards mem map add/remove against work thread
+amd::Monitor GraphNode::WorkerThreadLock_{};
 
 hipError_t GraphMemcpyNode1D::ValidateParams(void* dst, const void* src, size_t count,
                                                 hipMemcpyKind kind) {
@@ -87,6 +94,7 @@ hipError_t GraphMemcpyNode1D::ValidateParams(void* dst, const void* src, size_t
   return hipSuccess;
 }
 
+// ================================================================================================
 hipError_t GraphMemcpyNode::ValidateParams(const hipMemcpy3DParms* pNodeParams) {
   hipError_t status;
   status = ihipMemcpy3D_validate(pNodeParams);
@@ -102,6 +110,7 @@ hipError_t GraphMemcpyNode::ValidateParams(const hipMemcpy3DParms* pNodeParams)
   return hipSuccess;
 }
 
+// ================================================================================================
 bool Graph::isGraphValid(Graph* pGraph) {
   amd::ScopedLock lock(graphSetLock_);
   if (graphSet_.find(pGraph) == graphSet_.end()) {
@@ -110,6 +119,7 @@ bool Graph::isGraphValid(Graph* pGraph) {
   return true;
 }
 
+// ================================================================================================
 void Graph::AddNode(const Node& node) {
   vertices_.emplace_back(node);
   ClPrint(amd::LOG_INFO, amd::LOG_CODE, "[hipGraph] Add %s(%p)",
@@ -117,11 +127,13 @@ void Graph::AddNode(const Node& node) {
   node->SetParentGraph(this);
 }
 
+// ================================================================================================
 void Graph::RemoveNode(const Node& node) {
   vertices_.erase(std::remove(vertices_.begin(), vertices_.end(), node), vertices_.end());
   delete node;
 }
 
+// ================================================================================================
 // root nodes are all vertices with 0 in-degrees
 std::vector<Node> Graph::GetRootNodes() const {
   std::vector<Node> roots;
@@ -135,6 +147,7 @@ std::vector<Node> Graph::GetRootNodes() const {
   return roots;
 }
 
+// ================================================================================================
 // leaf nodes are all vertices with 0 out-degrees
 std::vector<Node> Graph::GetLeafNodes() const {
   std::vector<Node> leafNodes;
@@ -146,6 +159,7 @@ std::vector<Node> Graph::GetLeafNodes() const {
   return leafNodes;
 }
 
+// ================================================================================================
 size_t Graph::GetLeafNodeCount() const {
   int numLeafNodes = 0;
   for (auto entry : vertices_) {
@@ -166,95 +180,21 @@ std::vector<std::pair<Node, Node>> Graph::GetEdges() const {
   return edges;
 }
 
-void Graph::GetRunListUtil(Node v, std::unordered_map<Node, bool>& visited,
-                               std::vector<Node>& singleList,
-                               std::vector<std::vector<Node>>& parallelLists,
-                               std::unordered_map<Node, std::vector<Node>>& dependencies) {
-  // Mark the current node as visited.
-  visited[v] = true;
-  singleList.push_back(v);
-  // Recurse for all the vertices adjacent to this vertex
-  for (auto& adjNode : v->GetEdges()) {
-    if (!visited[adjNode]) {
-      // For the parallel list nodes add parent as the dependency
-      if (singleList.empty()) {
-        ClPrint(amd::LOG_INFO, amd::LOG_CODE,
-                "[hipGraph] For %s(%p) - add parent as dependency %s(%p)",
-                GetGraphNodeTypeString(adjNode->GetType()), adjNode,
-                GetGraphNodeTypeString(v->GetType()), v);
-        dependencies[adjNode].push_back(v);
-      }
-      GetRunListUtil(adjNode, visited, singleList, parallelLists, dependencies);
-    } else {
-      for (auto& list : parallelLists) {
-        // Merge singleList when adjNode matches with the first element of the list in existing
-        // lists
-        if (adjNode == list[0]) {
-          for (auto k = singleList.rbegin(); k != singleList.rend(); ++k) {
-            list.insert(list.begin(), *k);
-          }
-          singleList.erase(singleList.begin(), singleList.end());
-        }
-      }
-      // If the list cannot be merged with the existing list add as dependancy
-      if (!singleList.empty()) {
-        ClPrint(amd::LOG_INFO, amd::LOG_CODE, "[hipGraph] For %s(%p) - add dependency %s(%p)",
-                GetGraphNodeTypeString(adjNode->GetType()), adjNode,
-                GetGraphNodeTypeString(v->GetType()), v);
-        dependencies[adjNode].push_back(v);
-      }
-    }
-  }
-  if (!singleList.empty()) {
-    parallelLists.push_back(singleList);
-    singleList.erase(singleList.begin(), singleList.end());
-  }
-}
-// The function to do Topological Sort.
-// It uses recursive GetRunListUtil()
-void Graph::GetRunList(std::vector<std::vector<Node>>& parallelLists,
-                           std::unordered_map<Node, std::vector<Node>>& dependencies) {
-  std::vector<Node> singleList;
-
-  // Mark all the vertices as not visited
-  std::unordered_map<Node, bool> visited;
-  for (auto node : vertices_) visited[node] = false;
-
-  // Call the recursive helper function for all vertices one by one
-  for (auto node : vertices_) {
-    // If the node has embedded child graph
-    node->GetRunList(parallelLists, dependencies);
-    if (visited[node] == false) {
-      GetRunListUtil(node, visited, singleList, parallelLists, dependencies);
-    }
-  }
-  for (size_t i = 0; i < parallelLists.size(); i++) {
-    for (size_t j = 0; j < parallelLists[i].size(); j++) {
-      ClPrint(amd::LOG_INFO, amd::LOG_CODE, "[hipGraph] List %d - %s(%p)", i + 1,
-              GetGraphNodeTypeString(parallelLists[i][j]->GetType()), parallelLists[i][j]);
-    }
-  }
-}
-
 // ================================================================================================
 void Graph::ScheduleOneNode(Node node, int stream_id) {
   if (node->stream_id_ == -1) {
     // Assign active stream to the current node
     node->stream_id_ = stream_id;
-    max_streams_ = std::max(max_streams_, stream_id);
+    max_streams_ = std::max(max_streams_, (stream_id + 1));
 
-    // Update the dependencies if a signal is required
-    for (auto dep: node->GetDependencies()) {
-      // Check if the stream ID doesn't match and enable signal
-      if (dep->stream_id_ != node->stream_id_) {
-        dep->signal_is_required_ |= true;
-      }
-    }
     // Process child graph separately, since, there is no connection
     if (node->GetType() == hipGraphNodeTypeGraph) {
-      auto child = reinterpret_cast<hip::ChildGraphNode*>(node)->childGraph_;
+      auto child = reinterpret_cast<hip::ChildGraphNode*>(node)->GetChildGraph();
       child->ScheduleNodes();
       max_streams_ = std::max(max_streams_, child->max_streams_);
+      if (child->max_streams_ == 1) {
+        reinterpret_cast<hip::ChildGraphNode*>(node)->TopologicalOrder();
+      }
     }
     for (auto edge: node->GetEdges()) {
       ScheduleOneNode(edge, stream_id);
@@ -297,6 +237,14 @@ bool Graph::TopologicalOrder(std::vector<Node>& TopoOrder) {
   std::queue<Node> q;
   std::unordered_map<Node, int> inDegree;
   for (auto entry : vertices_) {
+    // Update the dependencies if a signal is required
+    for (auto dep: entry->GetDependencies()) {
+      // Check if the stream ID doesn't match and enable signal
+      if (dep->stream_id_ != entry->stream_id_) {
+        dep->signal_is_required_ = true;
+      }
+    }
+
     if (entry->GetInDegree() == 0) {
       q.push(entry);
     }
@@ -320,6 +268,7 @@ bool Graph::TopologicalOrder(std::vector<Node>& TopoOrder) {
   return false;
 }
 
+// ================================================================================================
 Graph* Graph::clone(std::unordered_map<Node, Node>& clonedNodes) const {
   Graph* newGraph = new Graph(device_, this);
   for (auto entry : vertices_) {
@@ -355,15 +304,19 @@ Graph* Graph::clone(std::unordered_map<Node, Node>& clonedNodes) const {
     userObj.first->owning_graphs_.insert(newGraph);
   }
   // Clone the root nodes to the new graph
-  memcpy(&newGraph->roots_[0], &roots_[0], sizeof(Node) * roots_.size());
+  if (roots_.size() > 0) {
+    memcpy(&newGraph->roots_[0], &roots_[0], sizeof(Node) * roots_.size());
+  }
   return newGraph;
 }
 
+// ================================================================================================
 Graph* Graph::clone() const {
   std::unordered_map<Node, Node> clonedNodes;
   return clone(clonedNodes);
 }
 
+// ================================================================================================
 bool GraphExec::isGraphExecValid(GraphExec* pGraphExec) {
   amd::ScopedLock lock(graphExecSetLock_);
   if (graphExecSet_.find(pGraphExec) == graphExecSet_.end()) {
@@ -372,6 +325,7 @@ bool GraphExec::isGraphExecValid(GraphExec* pGraphExec) {
   return true;
 }
 
+// ================================================================================================
 hipError_t GraphExec::CreateStreams(uint32_t num_streams) {
   parallel_streams_.reserve(num_streams);
   for (uint32_t i = 0; i < num_streams; ++i) {
@@ -395,18 +349,8 @@ hipError_t GraphExec::CreateStreams(uint32_t num_streams) {
 // ================================================================================================
 hipError_t GraphExec::Init() {
   hipError_t status = hipSuccess;
-  size_t min_num_streams = 1;
-  if ((DEBUG_HIP_FORCE_GRAPH_QUEUES == 0) || (parallelLists_.size() == 1)) {
-    for (auto& node : topoOrder_) {
-      status = node->GetNumParallelStreams(min_num_streams);
-      if (status != hipSuccess) {
-        return status;
-      }
-    }
-    status = CreateStreams(parallelLists_.size() - 1 + min_num_streams);
-  } else {
-    status = CreateStreams(clonedGraph_->max_streams_);
-  }
+  // create extra stream to avoid queue collision with the default execution stream
+  status = CreateStreams(clonedGraph_->max_streams_);
   if (status != hipSuccess) {
     return status;
   }
@@ -421,58 +365,46 @@ hipError_t GraphExec::Init() {
 //! Chunk size to add to kern arg pool
 constexpr uint32_t kKernArgChunkSize = 128 * Ki;
 // ================================================================================================
-void GetKernelArgSizeForGraph(std::vector<std::vector<Node>>& parallelLists,
-                              size_t& kernArgSizeForGraph) {
+void GraphExec::GetKernelArgSizeForGraph(size_t& kernArgSizeForGraph) {
   // GPU packet capture is enabled for kernel nodes. Calculate the kernel
   // arg size required for all graph kernel nodes to allocate
-  for (const auto& list : parallelLists) {
-    for (hip::GraphNode* node : list) {
-      if (node->GraphCaptureEnabled()) {
-        kernArgSizeForGraph += node->GetKerArgSize();
-      } else if (node->GetType() == hipGraphNodeTypeGraph) {
-        auto& childParallelLists =
-            reinterpret_cast<hip::ChildGraphNode*>(node)->GetParallelLists();
-        if (childParallelLists.size() == 1) {
-          GetKernelArgSizeForGraph(childParallelLists, kernArgSizeForGraph);
-        }
+  for (hip::GraphNode* node : topoOrder_) {
+    if (node->GraphCaptureEnabled()) {
+      kernArgSizeForGraph += node->GetKerArgSize();
+    } else if (node->GetType() == hipGraphNodeTypeGraph) {
+      auto childNode = reinterpret_cast<hip::ChildGraphNode*>(node);
+      // Child graph shares same kernel arg manager
+      GraphKernelArgManager* KernelArgManager = GetKernelArgManager();
+      KernelArgManager->retain();
+      childNode->graphExec_.SetKernelArgManager(KernelArgManager);
+      // Set capture stream for child graph
+      childNode->graphExec_.capture_stream_ = capture_stream_;
+      if (childNode->GetChildGraph()->max_streams_ == 1) {
+        childNode->graphExec_.GetKernelArgSizeForGraph(kernArgSizeForGraph);
       }
     }
   }
 }
 
 // ================================================================================================
-hipError_t AllocKernelArgForGraphNode(std::vector<hip::Node>& topoOrder,
-                                      hip::Stream* capture_stream, hip::GraphExec* graphExec) {
+hipError_t GraphExec::AllocKernelArgForGraphNode() {
   hipError_t status = hipSuccess;
-  for (auto& node : topoOrder) {
+  for (auto& node : topoOrder_) {
     if (node->GetType() == hipGraphNodeTypeKernel) {
       // Check if graph requires hidden heap and set as part of graphExec param.
       static bool initialized = false;
       if (!initialized && reinterpret_cast<hip::GraphKernelNode*>(node)->HasHiddenHeap()) {
-        graphExec->SetHiddenHeap();
+        SetHiddenHeap();
         initialized = true;
       }
     }
     if (node->GraphCaptureEnabled()) {
-      // From the kernel pool allocate the kern arg size required for the current node.
-      address kernArgOffset = nullptr;
-      if (node->GetKernargSegmentByteSize()) {
-        kernArgOffset = graphExec->kernArgManager_->AllocKernArg(
-            node->GetKernargSegmentByteSize(), node->GetKernargSegmentAlignment());
-        if (kernArgOffset == nullptr) {
-          return hipErrorMemoryAllocation;
-        }
-      }
-      // Form GPU packet capture for the kernel node.
-      node->CaptureAndFormPacket(capture_stream, kernArgOffset);
+      node->CaptureAndFormPacket(capture_stream_, GetKernelArgManager());
     } else if (node->GetType() == hipGraphNodeTypeGraph) {
       auto childNode = reinterpret_cast<hip::ChildGraphNode*>(node);
-      auto& childParallelLists = childNode->GetParallelLists();
-      if (childParallelLists.size() == 1) {
+      if (childNode->GetChildGraph()->max_streams_ == 1) {
         childNode->SetGraphCaptureStatus(true);
-        status =
-            AllocKernelArgForGraphNode(childNode->GetChildGraphNodeOrder(),
-                                       capture_stream, graphExec);
+        status = childNode->graphExec_.AllocKernelArgForGraphNode();
         if (status != hipSuccess) {
           return status;
         }
@@ -485,9 +417,9 @@ hipError_t AllocKernelArgForGraphNode(std::vector<hip::Node>& topoOrder,
 // ================================================================================================
 hipError_t GraphExec::CaptureAQLPackets() {
   hipError_t status = hipSuccess;
-  if (parallelLists_.size() == 1) {
+  if (clonedGraph_->max_streams_ == 1) {
     size_t kernArgSizeForGraph = 0;
-    GetKernelArgSizeForGraph(parallelLists_, kernArgSizeForGraph);
+    GetKernelArgSizeForGraph(kernArgSizeForGraph);
     auto device = g_devices[ihipGetDevice()]->devices()[0];
     // Add a larger initial pool to accomodate for any updates to kernel args
     bool bStatus = kernArgManager_->AllocGraphKernargPool(kernArgSizeForGraph + kKernArgChunkSize);
@@ -495,7 +427,7 @@ hipError_t GraphExec::CaptureAQLPackets() {
       return hipErrorMemoryAllocation;
     }
 
-    status = AllocKernelArgForGraphNode(topoOrder_, capture_stream_, this);
+    status = AllocKernelArgForGraphNode();
     if (status != hipSuccess) {
       return status;
     }
@@ -505,107 +437,14 @@ hipError_t GraphExec::CaptureAQLPackets() {
 }
 
 // ================================================================================================
-hipError_t GraphExec::UpdateAQLPacket(hip::GraphKernelNode* node) {
+hipError_t GraphExec::UpdateAQLPacket(hip::GraphNode* node) {
   hipError_t status = hipSuccess;
-  if (parallelLists_.size() == 1) {
-    size_t pool_new_usage = 0;
-    address kernArgOffset = nullptr;
-    kernArgOffset =
-        kernArgManager_->AllocKernArg(node->GetKerArgSize(), node->GetKernargSegmentAlignment());
-    if (kernArgOffset == nullptr) {
-      return hipErrorMemoryAllocation;
-    }
-    node->CaptureAndFormPacket(capture_stream_, kernArgOffset);
+  if (clonedGraph_->max_streams_ == 1) {
+    node->CaptureAndFormPacket(capture_stream_, kernArgManager_);
   }
   return hipSuccess;
 }
 
-hipError_t FillCommands(std::vector<std::vector<Node>>& parallelLists,
-                        std::unordered_map<Node, std::vector<Node>>& nodeWaitLists,
-                        std::vector<Node>& topoOrder, Graph* clonedGraph,
-                        amd::Command*& graphStart, amd::Command*& graphEnd, hip::Stream* stream) {
-  hipError_t status = hipSuccess;
-  for (auto& node : topoOrder) {
-    // TODO: clone commands from next launch
-    status = node->CreateCommand(node->GetQueue());
-    if (status != hipSuccess) return status;
-    amd::Command::EventWaitList waitList;
-    for (auto depNode : nodeWaitLists[node]) {
-      for (auto command : depNode->GetCommands()) {
-        waitList.push_back(command);
-      }
-    }
-    node->UpdateEventWaitLists(waitList);
-  }
-
-  std::vector<Node> rootNodes = clonedGraph->GetRootNodes();
-  ClPrint(amd::LOG_INFO, amd::LOG_CODE,
-          "[hipGraph] RootCommand get launched on stream %p", stream);
-
-  for (auto& root : rootNodes) {
-    //If rootnode is launched on to the same stream dont add dependency
-    if (root->GetQueue() != stream) {
-      if (graphStart == nullptr) {
-        graphStart = new amd::Marker(*stream, false, {});
-        if (graphStart == nullptr) {
-          return hipErrorOutOfMemory;
-        }
-      }
-      amd::Command::EventWaitList waitList;
-      waitList.push_back(graphStart);
-      auto commands = root->GetCommands();
-      if (!commands.empty()) {
-        commands[0]->updateEventWaitList(waitList);
-      }
-    }
-  }
-
-  // graphEnd ensures next enqueued ones start after graph is finished (all parallel branches)
-  amd::Command::EventWaitList graphLastCmdWaitList;
-  std::vector<Node> leafNodes = clonedGraph->GetLeafNodes();
-
-  for (auto& leaf : leafNodes) {
-    // If leaf node is launched on to the same stream dont add dependency
-    if (leaf->GetQueue() != stream) {
-      amd::Command::EventWaitList waitList;
-      waitList.push_back(graphEnd);
-      auto commands = leaf->GetCommands();
-      if (!commands.empty()) {
-        graphLastCmdWaitList.push_back(commands.back());
-      }
-    }
-  }
-  if (!graphLastCmdWaitList.empty()) {
-    graphEnd = new amd::Marker(*stream, false, graphLastCmdWaitList);
-    ClPrint(amd::LOG_INFO, amd::LOG_CODE,
-            "[hipGraph] EndCommand will get launched on stream %p", stream);
-    if (graphEnd == nullptr) {
-      graphStart->release();
-      return hipErrorOutOfMemory;
-    }
-  }
-  return hipSuccess;
-}
-
-void UpdateStream(std::vector<std::vector<Node>>& parallelLists, hip::Stream* stream,
-                 GraphExec* ptr) {
-  int i = 0;
-  for (const auto& list : parallelLists) {
-    // first parallel list will be launched on the same queue as parent
-    if (i == 0) {
-      for (auto& node : list) {
-        node->SetStream(stream, ptr);
-      }
-    } else {  // New stream for parallel branches
-      hip::Stream* stream = ptr->GetAvailableStreams();
-      for (auto& node : list) {
-        node->SetStream(stream, ptr);
-      }
-    }
-    i++;
-  }
-}
-
 // ================================================================================================
 
 void GraphExec::DecrementRefCount(cl_event event, cl_int command_exec_status, void* user_data) {
@@ -615,8 +454,7 @@ void GraphExec::DecrementRefCount(cl_event event, cl_int command_exec_status, vo
 
 // ================================================================================================
 
-hipError_t EnqueueGraphWithSingleList(std::vector<hip::Node>& topoOrder, hip::Stream* hip_stream,
-                                      hip::GraphExec* graphExec) {
+hipError_t GraphExec::EnqueueGraphWithSingleList(hip::Stream* hip_stream) {
   // Accumulate command tracks all the AQL packet batch that we submit to the HW. For now
   // we track only kernel nodes.
   amd::AccumulateCommand* accumulate = nullptr;
@@ -624,17 +462,18 @@ hipError_t EnqueueGraphWithSingleList(std::vector<hip::Node>& topoOrder, hip::St
   if (DEBUG_CLR_GRAPH_PACKET_CAPTURE) {
     accumulate = new amd::AccumulateCommand(*hip_stream, {}, nullptr);
   }
-  for (int i = 0; i < topoOrder.size(); i++) {
-    if (topoOrder[i]->GraphCaptureEnabled()) {
-      if (topoOrder[i]->GetEnabled()) {
-        hip_stream->vdev()->dispatchAqlPacket(topoOrder[i]->GetAqlPacket(),
-                                              topoOrder[i]->GetKernelName(),
-                                              accumulate);
+  for (int i = 0; i < topoOrder_.size(); i++) {
+    if (topoOrder_[i]->GraphCaptureEnabled()) {
+      if (topoOrder_[i]->GetEnabled()) {
+        std::vector<uint8_t*>& gpuPackets = topoOrder_[i]->GetAqlPackets();
+        for (auto& packet : gpuPackets) {
+          hip_stream->vdev()->dispatchAqlPacket(packet, topoOrder_[i]->GetKernelName(), accumulate);
+        }
       }
     } else {
-      topoOrder[i]->SetStream(hip_stream, graphExec);
-      status = topoOrder[i]->CreateCommand(topoOrder[i]->GetQueue());
-      topoOrder[i]->EnqueueCommands(hip_stream);
+      topoOrder_[i]->SetStream(hip_stream);
+      status = topoOrder_[i]->CreateCommand(topoOrder_[i]->GetQueue());
+      topoOrder_[i]->EnqueueCommands(hip_stream);
     }
   }
 
@@ -646,20 +485,24 @@ hipError_t EnqueueGraphWithSingleList(std::vector<hip::Node>& topoOrder, hip::St
 }
 
 // ================================================================================================
-void Graph::UpdateStreams(
-    hip::Stream* launch_stream,
-    const std::vector<hip::Stream*>& parallel_streams) {
+void Graph::UpdateStreams(hip::Stream* launch_stream,
+                          const std::vector<hip::Stream*>& parallel_streams) {
   // Allocate array for parallel streams, based on the graph scheduling + current stream
-  streams_.resize(parallel_streams.size() + 1);
-
+  // We create extra stream to avoid collision
+  streams_.resize(parallel_streams.size());
   // Current stream is the default in the assignment
   streams_[0] = launch_stream;
   // Assign the streams in the array of all streams
-  for (uint32_t i = 0; i < parallel_streams.size(); ++i) {
-    streams_[i + 1] = parallel_streams[i];
+  // Avoid stream that has collision with launch stream
+  for (uint32_t i = 1, j = 0; i < streams_.size(); j++) {
+    assert(j != parallel_streams.size());
+    if (launch_stream->getQueueID() != parallel_streams[j]->getQueueID()) {
+      streams_[i++] = parallel_streams[j];
+    }
   }
 }
 
+
 // ================================================================================================
 bool Graph::RunOneNode(Node node, bool wait) {
   if (node->launch_id_ == -1) {
@@ -687,24 +530,24 @@ bool Graph::RunOneNode(Node node, bool wait) {
       }
     }
 
-    if (node->GetType() == hipGraphNodeTypeGraph) {
-      // Process child graph separately, since, there is no connection
-      auto child = reinterpret_cast<hip::ChildGraphNode*>(node)->childGraph_;
-      if (!reinterpret_cast<hip::ChildGraphNode*>(node)->graphCaptureStatus_) {
-        child->RunNodes(node->stream_id_, &streams_);
-      }
-    } else {
-      // Create a wait list from the last launches of all dependencies
-      for (auto dep : wait_order_) {
-        if (dep != nullptr) {
-          // Add all commands in the wait list
-          if (dep->GetType() != hipGraphNodeTypeGraph) {
-            for (auto command : dep->GetCommands()) {
-              waitList.push_back(command);
-            }
+    // Create a wait list from the last launches of all dependencies
+    for (auto dep : wait_order_) {
+      if (dep != nullptr) {
+        // Add all commands in the wait list
+        if (dep->GetType() != hipGraphNodeTypeGraph) {
+          for (auto command : dep->GetCommands()) {
+            waitList.push_back(command);
           }
         }
       }
+    }
+    if (node->GetType() == hipGraphNodeTypeGraph) {
+      // Process child graph separately, since, there is no connection
+      auto child = reinterpret_cast<hip::ChildGraphNode*>(node)->GetChildGraph();
+      if (!reinterpret_cast<hip::ChildGraphNode*>(node)->graphCaptureStatus_) {
+        child->RunNodes(node->stream_id_, &streams_, &waitList);
+      }
+    } else {
       // Assing a stream to the current node
       node->SetStream(streams_);
       // Create the execution commands on the assigned stream
@@ -752,11 +595,21 @@ bool Graph::RunOneNode(Node node, bool wait) {
 // ================================================================================================
 bool Graph::RunNodes(
     int32_t base_stream,
-    const std::vector<hip::Stream*>* parallel_streams) {
+    const std::vector<hip::Stream*>* parallel_streams,
+    const amd::Command::EventWaitList* parent_waitlist) {
+
   if (parallel_streams != nullptr) {
     streams_ = *parallel_streams;
   }
 
+  // childgraph node has dependencies on parent graph nodes from other streams
+  if (parent_waitlist != nullptr) {
+    auto start_marker = new amd::Marker(*streams_[base_stream], true, *parent_waitlist);
+    if (start_marker != nullptr) {
+      start_marker->enqueue();
+      start_marker->release();
+    }
+  }
   amd::Command::EventWaitList wait_list;
   current_id_ = 0;
   memset(&leafs_[0], 0, sizeof(Node) * leafs_.size());
@@ -830,23 +683,20 @@ hipError_t GraphExec::Run(hipStream_t graph_launch_stream) {
   if (flags_ & hipGraphInstantiateFlagAutoFreeOnLaunch) {
     if (!topoOrder_.empty()) {
       topoOrder_[0]->GetParentGraph()->FreeAllMemory(launch_stream);
+      topoOrder_[0]->GetParentGraph()->memalloc_nodes_ = 0;
     }
   }
 
   // If this is a repeat launch, make sure corresponding MemFreeNode exists for a MemAlloc node
   if (repeatLaunch_ == true) {
-    for (auto& node : topoOrder_) {
-      if (node->GetType() == hipGraphNodeTypeMemAlloc &&
-          static_cast<GraphMemAllocNode*>(node)->IsActiveMem() == true) {
-        return hipErrorInvalidValue;
-      }
+    if (!topoOrder_.empty() && topoOrder_[0]->GetParentGraph()->GetMemAllocNodeCount() > 0) {
+       return hipErrorInvalidValue;
     }
-  } else {
+  }  else {
     repeatLaunch_ = true;
   }
 
-  if (parallelLists_.size() == 1 &&
-      instantiateDeviceId_ == launch_stream->DeviceId()) {
+  if (clonedGraph_->max_streams_ == 1 && instantiateDeviceId_ == launch_stream->DeviceId()) {
     if (DEBUG_CLR_GRAPH_PACKET_CAPTURE) {
       // If the graph has kernels that does device side allocation,  during packet capture, heap is
       // allocated because heap pointer has to be added to the AQL packet, and initialized during
@@ -857,43 +707,20 @@ hipError_t GraphExec::Run(hipStream_t graph_launch_stream) {
         initialized = true;
       }
     }
-    status = EnqueueGraphWithSingleList(topoOrder_, launch_stream, this);
-  } else if (parallelLists_.size() == 1 &&
-             instantiateDeviceId_ != launch_stream->DeviceId()) {
+    status = EnqueueGraphWithSingleList(launch_stream);
+  } else if (clonedGraph_->max_streams_ == 1 && instantiateDeviceId_ != launch_stream->DeviceId()) {
     for (int i = 0; i < topoOrder_.size(); i++) {
-      topoOrder_[i]->SetStream(launch_stream, this);
+      topoOrder_[i]->SetStream(launch_stream);
       status = topoOrder_[i]->CreateCommand(topoOrder_[i]->GetQueue());
       topoOrder_[i]->EnqueueCommands(launch_stream);
     }
   } else {
-    if (DEBUG_HIP_FORCE_GRAPH_QUEUES == 0) {
-      UpdateStream(parallelLists_, launch_stream, this);
-      amd::Command* rootCommand = nullptr;
-      amd::Command* endCommand = nullptr;
-      status = FillCommands(parallelLists_, nodeWaitLists_, topoOrder_, clonedGraph_, rootCommand,
-                            endCommand, launch_stream);
-      if (status != hipSuccess) {
-        return status;
-      }
-      if (rootCommand != nullptr) {
-        rootCommand->enqueue();
-        rootCommand->release();
-      }
-      for (int i = 0; i < topoOrder_.size(); i++) {
-        topoOrder_[i]->EnqueueCommands(topoOrder_[i]->GetQueue());
-      }
-      if (endCommand != nullptr) {
-        endCommand->enqueue();
-        endCommand->release();
-      }
-    } else {
-      // Update streams for the graph execution
-      clonedGraph_->UpdateStreams(launch_stream, parallel_streams_);
-      // Execute all nodes in the graph
-      if (!clonedGraph_->RunNodes()) {
-        LogError("Failed to launch nodes!");
-        return hipErrorOutOfMemory;
-      }
+    // Update streams for the graph execution
+    clonedGraph_->UpdateStreams(launch_stream, parallel_streams_);
+    // Execute all nodes in the graph
+    if (!clonedGraph_->RunNodes()) {
+      LogError("Failed to launch nodes!");
+      return hipErrorOutOfMemory;
     }
   }
   this->retain();
