@@ -76,7 +76,10 @@ void** __hipRegisterFatBinary(const void* data) {
                    fbwrapper->version);
     return nullptr;
   }
-  return reinterpret_cast<void**>(PlatformState::instance().addFatBinary(fbwrapper->binary));
+
+  bool success{};
+  auto fat_binary_info = PlatformState::instance().addFatBinary(fbwrapper->binary, success);
+  return success ? reinterpret_cast<void**>(fat_binary_info) : nullptr;
 }
 
 void __hipRegisterFunction(hip::FatBinaryInfo** modules, const void* hostFunction,
@@ -182,14 +185,10 @@ void __hipRegisterTexture(
 }
 
 void __hipUnregisterFatBinary(hip::FatBinaryInfo** modules) {
+  // By calling hipDeviceSynchronize ensure that all HSA signal handlers
+  // complete before removeFatBinary
   static std::once_flag unregister_device_sync;
-  std::call_once(unregister_device_sync, [](){
-    for (auto& hipDevice : g_devices) {
-      // By synchronizing devices ensure that all HSA signal handlers
-      // complete before removeFatBinary
-      hipDevice->SyncAllStreams(true);
-    }
-  });
+  std::call_once(unregister_device_sync, hipDeviceSynchronize);
   hipError_t err = PlatformState::instance().removeFatBinary(modules);
   guarantee((err == hipSuccess), "Cannot Unregister Fat Binary, error:%d", err);
 }
@@ -283,6 +282,9 @@ hipError_t hipLaunchByPtr(const void* hostFunction) {
   void* extra[] = {HIP_LAUNCH_PARAM_BUFFER_POINTER, &exec.arguments_[0],
                    HIP_LAUNCH_PARAM_BUFFER_SIZE, &size, HIP_LAUNCH_PARAM_END};
 
+  STREAM_CAPTURE(hipLaunchByPtr, exec.hStream_, func, exec.blockDim_, exec.gridDim_,
+                 exec.sharedMem_, extra);
+
   HIP_RETURN(hipModuleLaunchKernel(func, exec.gridDim_.x, exec.gridDim_.y, exec.gridDim_.z,
                                    exec.blockDim_.x, exec.blockDim_.y, exec.blockDim_.z,
                                    exec.sharedMem_, exec.hStream_, nullptr, extra));
@@ -615,6 +617,9 @@ hipError_t hipOccupancyMaxActiveBlocksPerMultiprocessorWithFlags(int* numBlocks,
 hipError_t ihipLaunchKernel(const void* hostFunction, dim3 gridDim, dim3 blockDim, void** args,
                             size_t sharedMemBytes, hipStream_t stream, hipEvent_t startEvent,
                             hipEvent_t stopEvent, int flags) {
+  if (!hip::isValid(stream)) {
+    return hipErrorInvalidValue;
+  }
   hipFunction_t func = nullptr;
   int deviceId = hip::Stream::DeviceId(stream);
   hipError_t hip_error = PlatformState::instance().getStatFunc(&func, hostFunction, deviceId);
@@ -625,6 +630,13 @@ hipError_t ihipLaunchKernel(const void* hostFunction, dim3 gridDim, dim3 blockDi
       return hipErrorInvalidDeviceFunction;
     }
   }
+
+  constexpr auto gridDimYZmax = static_cast<uint64_t>(std::numeric_limits<uint16_t>::max()) + 1;
+  const auto& isa = g_devices[deviceId]->devices()[0]->isa().versionMajor();
+  if (isa >= 12 && (gridDim.y > gridDimYZmax || gridDim.z > gridDimYZmax)) {
+    return hipErrorInvalidConfiguration;
+  }
+
   size_t globalWorkSizeX = static_cast<size_t>(gridDim.x) * blockDim.x;
   size_t globalWorkSizeY = static_cast<size_t>(gridDim.y) * blockDim.y;
   size_t globalWorkSizeZ = static_cast<size_t>(gridDim.z) * blockDim.z;
@@ -796,6 +808,12 @@ hipError_t PlatformState::getDynFunc(hipFunction_t* hfunc, hipModule_t hmod,
   return it->second->getDynFunc(hfunc, func_name);
 }
 
+bool PlatformState::isValidDynFunc(const void* hfunc) {
+  amd::ScopedLock lock(lock_);
+  return std::any_of(dynCO_map_.begin(), dynCO_map_.end(),
+                     [&](auto& it) { return it.second->isValidDynFunc(hfunc); });
+}
+
 hipError_t PlatformState::getDynGlobalVar(const char* hostVar, hipModule_t hmod,
                                           hipDeviceptr_t* dev_ptr, size_t* size_ptr) {
   amd::ScopedLock lock(lock_);
@@ -884,8 +902,8 @@ hipError_t PlatformState::digestFatBinary(const void* data, hip::FatBinaryInfo*&
   return statCO_.digestFatBinary(data, programs);
 }
 
-hip::FatBinaryInfo** PlatformState::addFatBinary(const void* data) {
-  return statCO_.addFatBinary(data, initialized_);
+hip::FatBinaryInfo** PlatformState::addFatBinary(const void* data, bool& success) {
+  return statCO_.addFatBinary(data, initialized_, success);
 }
 
 hipError_t PlatformState::removeFatBinary(hip::FatBinaryInfo** module) {
