@@ -31,7 +31,7 @@ namespace hip {
 Stream::Stream(hip::Device* dev, Priority p, unsigned int f, bool null_stream,
                const std::vector<uint32_t>& cuMask, hipStreamCaptureStatus captureStatus)
     : amd::HostQueue(*dev->asContext(), *dev->devices()[0], 0, amd::CommandQueue::RealTimeDisabled,
-        convertToQueuePriority(p), cuMask),
+                     convertToQueuePriority(p), cuMask),
       lock_("Stream Callback lock"),
       device_(dev),
       priority_(p),
@@ -40,10 +40,9 @@ Stream::Stream(hip::Device* dev, Priority p, unsigned int f, bool null_stream,
       cuMask_(cuMask),
       captureStatus_(captureStatus),
       originStream_(false),
-      captureID_(0)
-      {
-        device_->AddStream(this);
-      }
+      captureID_(0) {
+  device_->AddStream(this);
+}
 
 // ================================================================================================
 hipError_t Stream::EndCapture() {
@@ -74,10 +73,17 @@ bool Stream::Create() {
 
 // ================================================================================================
 void Stream::Destroy(hip::Stream* stream) {
+  stream->device().removeFromActiveQueues(stream);
   stream->device_->RemoveStream(stream);
   stream->release();
+  stream = nullptr;
 }
 
+// ================================================================================================
+bool Stream::terminate() {
+  HostQueue::terminate();
+  return true;
+}
 // ================================================================================================
 bool isValid(hipStream_t& stream) {
   // NULL stream is always valid
@@ -235,6 +241,12 @@ hipStream_t stream_per_thread::get() {
   return m_streams[currDev];
 }
 
+void stream_per_thread::clear_spt() {
+  if (!m_streams.empty()) {
+    m_streams[getCurrentDevice()->deviceId()] = nullptr;
+  }
+}
+
 
 // ================================================================================================
 void getStreamPerThread(hipStream_t& stream) {
@@ -359,7 +371,6 @@ hipError_t hipStreamSynchronize_common(hipStream_t stream) {
 
     // Wait for the current host queue
     hip_stream->finish();
-
     // Release freed memory for all memory pools on the device
     hip_stream->GetDevice()->ReleaseFreedMemory();
   }
@@ -477,8 +488,7 @@ hipError_t hipStreamWaitEvent_common(hipStream_t stream, hipEvent_t event, unsig
         // If stream is capturing but event is not recorded on event's stream.
         return hipErrorStreamCaptureIsolation;
       }
-      if ((waitStream != nullptr && stream != hipStreamLegacy) &&
-          (eventStream->DeviceId() == waitStream->DeviceId())) {
+      if ((waitStream != nullptr) && (eventStream->DeviceId() == waitStream->DeviceId())) {
         eventStream->GetDevice()->AddSafeStream(eventStream, waitStream);
       }
     }
