@@ -318,12 +318,11 @@ Command::Command(HostQueue& queue, cl_command_type type, const EventWaitList& ev
   }
 }
 
-SysmemPool<ComputeCommand> Command::command_pool_ ROCCLR_INIT_PRIORITY(101);
-
+SysmemPool<ComputeCommand>* Command::command_pool_ = new SysmemPool<ComputeCommand>;
 // ================================================================================================
 void Command::operator delete(void* ptr) {
   if (DEBUG_CLR_SYSMEM_POOL) {
-    command_pool_.Free(ptr);
+    command_pool_->Free(ptr);
   } else {
     ::operator delete (ptr);
   }
@@ -332,7 +331,7 @@ void Command::operator delete(void* ptr) {
 // ================================================================================================
 void* Command::operator new(size_t size) {
   if (DEBUG_CLR_SYSMEM_POOL) {
-    return command_pool_.Alloc(size);
+    return command_pool_->Alloc(size);
   } else {
     return ::operator new (size);
   }
