@@ -190,7 +190,7 @@ class Event : public RuntimeObject {
 
   //! Signal all threads waiting on this event.
   void signal() {
-    ScopedLock lock(lock_);
+    ScopedLock lock(lock_);// Unnecessary
     lock_.notifyAll();
   }
 
@@ -246,6 +246,12 @@ union CopyMetadata {
         copyEnginePreference_(copyEnginePreference) {}
 };
 
+// Interface to callback to allocate kernel args from the graph kernel arg pool.
+class GraphKernelArgManager {
+ public:
+  virtual address AllocKernArg(size_t size, size_t alignment) = 0;
+};
+
 /*! \brief An operation that is submitted to a command queue.
  *
  *  %Command is the abstract base type of all OpenCL operations
@@ -255,7 +261,7 @@ union CopyMetadata {
  */
 class Command : public Event {
  private:
-  static SysmemPool<ComputeCommand> command_pool_;  //!< Pool of active commands
+  static SysmemPool<ComputeCommand> *command_pool_;  //!< Pool of active commands
   HostQueue* queue_;               //!< The command queue this command is enqueue into
   Command* next_;                  //!< Next GPU command in the queue list
   Command* batch_head_ = nullptr;  //!< The head of the batch commands
@@ -264,7 +270,8 @@ class Command : public Event {
   const Event* waitingEvent_;  //!< Waiting event associated with the marker
 
   bool packetCapturing_ = false;           //!< Flag to enable/disable graph gpu packet capture
-  uint8_t* gpuPacket_ = nullptr;     //!< GPU packet to capture, when graph capturing is enabled
+  std::vector<uint8_t*>* gpuPackets_;  //!< GPU packets captured when graph capturing is enabled
+  GraphKernelArgManager* graphKernArgMgr_ = nullptr;  //!< KernelMgr for graph
   address kernArgOffset_ = nullptr;  //!< KernelArg buffer to used when graph capturing is enabled
   std::string* capturedKernelName_ = nullptr;  //!< Kenrnel under capture
  protected:
@@ -304,14 +311,21 @@ class Command : public Event {
 
  public:
   //! Returns AQL buffer state
+  static void ReleaseSysmemPool() {
+    if (command_pool_ != nullptr) {
+      delete command_pool_;
+      command_pool_ = nullptr;
+    }
+  }
   bool getPktCapturingState() const { return packetCapturing_; }
 
   //! Sets AQL capture state, aql packet to capture and where to copy kernArgs
-  void setPktCapturingState(bool state, uint8_t* packet, address kernArgOffset,
-                            std::string* capturedKernelName) {
+  void setPktCapturingState(bool state, std::vector<uint8_t*>* packet,
+                         amd::GraphKernelArgManager* graphKernArgMgr,
+                         std::string* capturedKernelName) {
     packetCapturing_ = state;
-    gpuPacket_ = packet;
-    kernArgOffset_ = kernArgOffset;
+    gpuPackets_ = packet;
+    graphKernArgMgr_ = graphKernArgMgr;
     capturedKernelName_ = capturedKernelName;
   }
 
@@ -322,11 +336,16 @@ class Command : public Event {
     }
   }
 
-  //! returns the graph executable object command belongs to.
-  const uint8_t* getAqlPacket() const { return gpuPacket_; }
+  //! Returns the graph executable object command belongs to.
+  const uint8_t* getAqlPacket() const {
+    uint8_t* packet = new uint8_t[64];
+    gpuPackets_->push_back(packet);
+    return packet;
+  }
 
-  //! returns the graph executable object command belongs to.
-  const address getKernArgOffset() const { return kernArgOffset_; }
+  address getKernArgOffset(int size, int alignment) {
+    return graphKernArgMgr_->AllocKernArg(size, alignment);
+  }
 
   //! Overload new/delete for fast commands allocation/destruction
   void* operator new(size_t size);
@@ -826,7 +845,7 @@ class FillMemoryCommand : public OneMemoryArgCommand {
 
 class StreamOperationCommand : public OneMemoryArgCommand {
  private:
-  uint64_t value_;       // !< Value to Wait on or to Write.
+  uint64_t value_;      // !< Value to Wait on or to Write.
   uint64_t mask_;       // !< Mask to be applied on signal value for Wait operation.
   unsigned int flags_;  // !< Flags defining the Wait condition.
   size_t offset_;       // !< Offset into memory for Write
@@ -869,6 +888,46 @@ class StreamOperationCommand : public OneMemoryArgCommand {
   const size_t sizeBytes() const { return sizeBytes_; }
 };
 
+/*! \brief      A batch memory operation command.
+ *
+ *  \details    Batch operations to synchronize the stream via memory operations
+ *              Operations are either 32-bit stream wait or write.
+ *              Wait: All the commands issued after stream wait are not executed
+ *              until the wait condition is true.
+ *              Write: Writes a 32 or 64 bit vaue to the memory using a GPU Blit.
+ *              The operations are enqueued in the order they appear in the array.
+ */
+
+class BatchMemoryOperationCommand : public Command {
+ public:
+  BatchMemoryOperationCommand(HostQueue& queue, cl_command_type cmdType, uint32_t count,
+                              uint32_t flags, EventWaitList& eventWaitList, const void* paramArray,
+                              size_t paramSize)
+      : Command(queue, cmdType, eventWaitList),
+        count_(count),
+        paramArray_(paramArray),
+        flags_(flags),
+        paramSize_(paramSize) {
+    // Sanity check
+    assert(((cmdType == ROCCLR_COMMAND_BATCH_STREAM)) && "Invalid batch memory operation");
+  }
+
+  virtual void submit(device::VirtualDevice& device) { device.submitBatchMemoryOperation(*this); }
+
+  //! Returns the value
+  const uint64_t count() const { return count_; }
+  //! Return the pointer to the paramList
+  const void* getParamPtr() { return paramArray_; }
+  //! Return the size of a single mem op param in bytes
+  const size_t paramSize() const { return paramSize_; }
+
+ private:
+  uint32_t count_;          // !< The number of operations in the array.
+  uint32_t flags_;          // !< Reserved for future expansion. Must be 0.
+  const void* paramArray_;  // !< Pointer to the array of individual operations
+  size_t paramSize_;        // !< size in bytes of the param array passed
+};
+
 /*! \brief      A generic copy memory command
  *
  *  \details    Used for both buffers and images. Backends are expected
@@ -1731,6 +1790,13 @@ class CopyMemoryP2PCommand : public CopyMemoryCommand {
       : CopyMemoryCommand(queue, cmdType, eventWaitList, srcMemory, dstMemory, srcOrigin, dstOrigin,
                           size) {}
 
+  CopyMemoryP2PCommand(HostQueue& queue, cl_command_type cmdType, const EventWaitList& eventWaitList,
+                    Memory& srcMemory, Memory& dstMemory, Coord3D srcOrigin, Coord3D dstOrigin,
+                    Coord3D size, const BufferRect& srcRect, const BufferRect& dstRect,
+                    amd::CopyMetadata copyMetadata = amd::CopyMetadata())
+      : CopyMemoryCommand(queue, cmdType, eventWaitList, srcMemory, dstMemory, srcOrigin, dstOrigin,
+                          size, srcRect, dstRect) {}
+
   virtual void submit(device::VirtualDevice& device) { device.submitCopyMemoryP2P(*this); }
 
   bool validateMemory();
@@ -1832,6 +1898,7 @@ union ComputeCommand {
   CopyMemoryP2PCommand          cmd25;
   SvmPrefetchAsyncCommand       cmd26;
   VirtualMapCommand             cmd27;
+  BatchMemoryOperationCommand   cmd28;
   ComputeCommand() {}
   ~ComputeCommand() {}
 };
